==> memx-0.1.1.crate: memx-0.1.1/benches/bench-memchr.rs <==
use criterion::{black_box, criterion_group, criterion_main, Criterion};

#[inline(never)]
fn process_std_memchr(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        for i in 0..line_len {
            if line_bytes[i] == pat_byte {
                found += 1;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr_basic(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr_basic(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr_libc(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr_libc(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

mod create_data;

fn criterion_benchmark(c: &mut Criterion) {
    let (v, pat_byte, match_cnt) = create_data::create_data_chr();
    let vv: Vec<&str> = v.iter().map(|item| item.as_str()).collect();
    //
    let n = process_std_memchr(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr_basic(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    //
    c.bench_function("std_memchr", |b| {
        b.iter(|| {
            let _r = process_std_memchr(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr", |b| {
        b.iter(|| {
            let _r = process_memx_memchr(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr_basic", |b| {
        b.iter(|| {
            let _r = process_memx_memchr_basic(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr_libc", |b| {
        b.iter(|| {
            let _r = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
        })
    });
}

criterion_group! {
    name = benches;
    config = Criterion::default()
        .warm_up_time(std::time::Duration::from_millis(300))
        .measurement_time(std::time::Duration::from_millis(3000));
    targets = criterion_benchmark
}
//criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);

==> memx-0.1.2.crate: memx-0.1.2/benches/bench-memchr.rs <==
use criterion::{black_box, criterion_group, criterion_main, Criterion};

#[inline(never)]
fn process_std_memchr(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        for i in 0..line_len {
            if line_bytes[i] == pat_byte {
                found += 1;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr_basic(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr_basic(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memx_memchr_libc(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memx::memchr_libc(&line_bytes[curr_idx..], pat_byte);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

#[inline(never)]
fn process_memchr_memchr(texts: &[&str], pat_byte: u8) -> usize {
    let mut found: usize = 0;
    for line in texts {
        let line_bytes = line.as_bytes();
        let line_len = line_bytes.len();
        let mut curr_idx = 0;
        while curr_idx < line_len {
            let r = memchr::memchr(pat_byte, &line_bytes[curr_idx..]);
            if let Some(pos) = r {
                found += 1;
                curr_idx = pos + curr_idx + 1;
            } else {
                break;
            }
        }
    }
    found
}

mod create_data;

fn criterion_benchmark(c: &mut Criterion) {
    let (v, pat_byte, match_cnt) = create_data::create_data_chr();
    let vv: Vec<&str> = v.iter().map(|item| item.as_str()).collect();
    //
    let n = process_std_memchr(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr_basic(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    let n = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
    assert_eq!(n, match_cnt);
    //
    c.bench_function("std_memchr", |b| {
        b.iter(|| {
            let _r = process_std_memchr(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr", |b| {
        b.iter(|| {
            let _r = process_memx_memchr(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr_basic", |b| {
        b.iter(|| {
            let _r = process_memx_memchr_basic(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memx_memchr_libc", |b| {
        b.iter(|| {
            let _r = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
        })
    });
    c.bench_function("memchr_memchr", |b| {
        b.iter(|| {
            let _r = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
        })
    });
}

criterion_group! {
    name = benches;
    config = Criterion::default()
        .warm_up_time(std::time::Duration::from_millis(300))
        .measurement_time(std::time::Duration::from_millis(3000));
    targets = criterion_benchmark
}
//criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);
